@@ -5,22 +5,13 @@
 //! and you can find how they're simulated at [Simulator::run](struct.Simulator.html#method.run)
 //!
 
-use std::sync::{Arc, Mutex};
 use std::time;
 
-const DATA_SIZE: usize = 0x0040_0000; // TODO: this, but I think it's about this much
-const MMIO_SIZE: usize = 0x0021_0000;
-const MMIO_START: usize = 0xff00_0000;
-const KBMMIO_CONTROL: usize = 0xff20_0000;
-const KBMMIO_DATA: usize = 0xff20_0004;
-
-use crate::renderer::{FRAME_0, FRAME_1, HEIGHT, WIDTH};
-const VIDEO_START: usize = MMIO_START + FRAME_0;
-const VIDEO_END: usize = MMIO_START + FRAME_1 + WIDTH * HEIGHT;
-
-const HEAP_START: usize = 0x1004_0000;
-
 use crate::parser::{self, Includable, MacroParseable, RISCVParser};
+use crate::renderer::{FRAME_0, FRAME_1, HEIGHT, WIDTH};
+
+mod memory;
+use memory::*;
 
 mod into_register;
 use into_register::*;
@@ -30,125 +21,6 @@ mod midi;
 
 mod util;
 
-use byteorder::{ByteOrder, LittleEndian};
-
-pub struct Memory {
-    pub mmio: Arc<Mutex<Vec<u8>>>,
-    data: Vec<u8>,
-
-    /// Memory allocated by `sbrk`
-    dynamic: Vec<u8>,
-}
-
-impl Memory {
-    pub fn new() -> Self {
-        Self {
-            mmio: Arc::new(Mutex::new(vec![0; MMIO_SIZE])),
-            data: vec![0; DATA_SIZE],
-            dynamic: vec![],
-        }
-    }
-
-    /// Sets N bytes in the video memory, but ignores bytes equal to 0xC7.
-    fn set_with_transparency(&mut self, i: usize, mut x: u32, n: usize) -> bool {
-        if i < VIDEO_START || i >= VIDEO_END {
-            return false;
-        }
-
-        let mut mmio = self.mmio.lock().unwrap();
-        let i = i - MMIO_START;
-
-        for data in &mut mmio[i..i + n] {
-            let byte = x as u8;
-            if byte != 0xC7 {
-                *data = byte;
-            }
-
-            x >>= 8;
-        }
-
-        true
-    }
-
-    pub fn get_with<T, F>(&self, i: usize, read: F) -> T
-    where
-        F: FnOnce(&[u8]) -> T,
-    {
-        if i >= MMIO_START {
-            // MMIO
-            let mut mmio = self.mmio.lock().unwrap();
-            if i == KBMMIO_DATA {
-                mmio[KBMMIO_CONTROL - MMIO_START] = 0;
-            }
-            read(&mmio[i - MMIO_START..])
-        } else if i >= HEAP_START {
-            // Heap/dynamic memory
-            read(&self.dynamic[i - HEAP_START..])
-        } else {
-            // Data memory
-            read(&self.data[i..])
-        }
-    }
-
-    pub fn set_with<T, F, R>(&mut self, i: usize, x: T, write: F) -> R
-    where
-        F: FnOnce(&mut [u8], T) -> R,
-    {
-        if i >= MMIO_START {
-            // MMIO
-            let mut mmio = self.mmio.lock().unwrap();
-            write(&mut mmio[i - MMIO_START..], x)
-        } else if i >= HEAP_START {
-            // Heap/dynamic memory
-            write(&mut self.dynamic[i - HEAP_START..], x)
-        } else {
-            // Data memory
-            write(&mut self.data[i..], x)
-        }
-    }
-
-    pub fn get_byte(&self, i: usize) -> u8 {
-        self.get_with(i, |v| v[0])
-    }
-
-    pub fn set_byte(&mut self, i: usize, x: u8) {
-        if self.set_with_transparency(i, x as u32, 1) {
-            return;
-        }
-        self.set_with(i, x, |v, x| v[0] = x)
-    }
-
-    pub fn get_half(&self, i: usize) -> u16 {
-        self.get_with(i, LittleEndian::read_u16)
-    }
-
-    pub fn set_half(&mut self, i: usize, x: u16) {
-        if self.set_with_transparency(i, x as u32, 2) {
-            return;
-        }
-        self.set_with(i, x, LittleEndian::write_u16)
-    }
-
-    pub fn get_word(&self, i: usize) -> u32 {
-        self.get_with(i, LittleEndian::read_u32)
-    }
-
-    pub fn set_word(&mut self, i: usize, x: u32) {
-        if self.set_with_transparency(i, x, 4) {
-            return;
-        }
-        self.set_with(i, x, LittleEndian::write_u32)
-    }
-
-    pub fn get_float(&self, i: usize) -> f32 {
-        self.get_with(i, LittleEndian::read_f32)
-    }
-
-    pub fn set_float(&mut self, i: usize, x: f32) {
-        self.set_with(i, x, LittleEndian::write_f32)
-    }
-}
-
 /// Returned by the [ecall](struct.Simulator.html#method.ecall) procedure
 enum EcallSignal {
     Nothing,
@@ -161,18 +33,19 @@ enum EcallSignal {
 pub struct Simulator {
     registers: [u32; 32],
     floats: [f32; 32],
-    status: Vec<u32>, // I'm not sure myself how many status register I'll use
+    status: Vec<u32>, // I'm not sure myself how many status registers I'll use
     pc: usize,
     started_at: time::Instant,
 
     open_files: files::FileHolder,
+    midi_player: midi::MidiPlayer,
 
     pub memory: Memory,
     pub code: Vec<parser::Instruction>,
 }
 
 impl Simulator {
-    pub fn new() -> Self {
+    pub fn new(midi_port: Option<usize>) -> Self {
         Self {
             registers: [0; 32],
             floats: [0.0; 32],
@@ -180,6 +53,7 @@ impl Simulator {
             pc: 0,
             started_at: time::Instant::now(), // Will be set again in run()
             open_files: files::FileHolder::new(),
+            midi_player: midi::MidiPlayer::new(midi_port),
             memory: Memory::new(),
             code: Vec::new(),
         }
@@ -583,7 +457,7 @@ impl Simulator {
             return EcallSignal::Nothing;
         }
 
-        if midi::handle_ecall(a7, &mut self.registers) {
+        if self.midi_player.handle_ecall(a7, &mut self.registers) {
             return EcallSignal::Nothing;
         }
 
@@ -596,6 +470,10 @@ impl Simulator {
                 // print int
                 print!("{}", self.get_reg::<i32>(10));
             }
+            2 => {
+                // print float
+                print!("{}", self.floats[10]);
+            }
             4 => {
                 // print string
                 let start = self.get_reg::<u32>(10) as usize; // a0
@@ -611,8 +489,10 @@ impl Simulator {
                 self.set_reg(10, buf.trim().parse::<i32>().unwrap());
             }
             6 => {
-                // print float
-                print!("{}", self.floats[10]);
+                // read float
+                let mut buf = String::new();
+                std::io::stdin().read_line(&mut buf).unwrap();
+                self.floats[10] = buf.trim().parse::<f32>().unwrap();
             }
 
             9 => {
@@ -649,10 +529,6 @@ impl Simulator {
                 self.set_reg(11, (ms >> 32) as u32);
             }
 
-            31 | 33 => {
-                // midi stuff, but nops for now
-            }
-
             32 => {
                 // sleep ms
                 let t = self.get_reg::<u32>(10);
@@ -721,3 +597,4 @@ impl Simulator {
         EcallSignal::Nothing
     }
 }
+
